// SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
// SPDX-FileCopyrightText: 2019-2025 Harald Sitter <sitter@kde.org>

#pragma once

#include <QString>

namespace Paths
{
[[nodiscard]] QString drkonqiExe();
} // namespace Paths
